@@ -615,7 +615,7 @@ class JsonRPCProtocol(JsonRPCProtocolBase):
                             if time.monotonic() - start > 120:
                                 raise TimeoutError("Can't set future result.")
 
-                            await asyncio.sleep(0.001)
+                            await asyncio.sleep(0)
 
                     else:
                         self._logger.warning(f"Response {entry:r} loop is not running.")
@@ -717,6 +717,12 @@ class JsonRPCProtocol(JsonRPCProtocolBase):
 
         def done(t: asyncio.Task[Any]) -> None:
             try:
+                if not t.cancelled():
+                    ex = t.exception()
+                    if ex is not None:
+                        self._logger.exception(ex, exc_info=ex)
+                        raise JsonRPCErrorException(JsonRPCErrors.INTERNAL_ERROR, f"{type(ex).__name__}: {ex}") from ex
+
                 self.send_response(message.id, t.result())
             except asyncio.CancelledError:
                 self._logger.debug(f"request message {repr(message)} canceled")
